<?php
// $Id: requests.inc,v 1.2 2010/10/21 21:38:52 tomdude48 Exp $

function linkintel_node_requests_page($node, $mode) {
  drupal_set_title('Requests');
  $kwresearch_exists = module_exists('kwresearch');
  
  if($kid) {
    $where[] = 'r.kid = "%s"';
    $params[] = $kid;
  }
  
  $header = array();
  $header[] = array('data' => t('Keyword'), 'field' => 'k.keyword');
  $header[] = array('data' => t('Score'), 'field' => 'score');
  $header[] = array('data' => t('Request'), 'field' => 'r.priority', 'sort' => 'desc');
  $header[] = array('data' => t('Page'), 'field' => 'p.priority');
  if($kwresearch_exists) {
    $header[] = array('data' => t('Keyword'), 'field' => 'keyword_priority'); 
  }  
  $header[] = array('data' => t('Title'));
  $header[] = array('data' => t('Sync'), 'field' => 'r.sync');
  $header[] = array('data' => t('Ops'));
  
  if($kwresearch_exists) {
    $select = "kwr.kid AS kwr_kid, kwr.priority AS keyword_priority, (r.priority * IF(p.priority >= 0, p.priority, 50) * IF(kwr.priority >= 0, kwr.priority, 50) / 1250) AS score";
    $join = "LEFT JOIN {kwresearch_keyword} kwr ON kwr.keyword = k.keyword";
  }
  else {
    $select = "(r.priority * IF(p.priority >= 0, p.priority, 50) / 25) AS score";
  }
  
  $sql = "
    SELECT r.*, IF(r.nid > 0, r.nid, r.path) AS pid, k.keyword, p.priority AS page_priority, 
    $select
    FROM {linkintel_request} r
    JOIN {linkintel_keyword} k ON k.kid = r.kid
    LEFT JOIN {linkintel_page} p ON p.nid = r.nid 
    $join
    WHERE r.nid = %d
  ";
  $tablesort = tablesort_sql($header);
  $sql .= $tablesort;
  $result = pager_query($sql, 100, 0, NULL, $node->nid);

  $priorities = linkintel_get_priority_options();
  $dest = 'node/' . $node->nid . '/linkintel/requests';
  $rows = array();
  while ($r = db_fetch_object($result)) {
    $attributes = unserialize($r->attributes);
    $ops = l(t('edit'), 'admin/content/linkintel/request/edit/' . $node->nid, array('query' => 'destination=' . $dest));
    $ops .= ' | ' . l(t('delete'), 'admin/content/linkintel/request/delete/' . $node->nid, array('query' => 'destination=' . $dest));
    $row = array();
    $row[] = check_plain($r->keyword);
    $row[] = number_format($r->score);
    $t = ($priorities[$r->priority]) ? $priorities[$r->priority] : $r->priority;
    $row[] = l($t, 'admin/content/linkintel/request/edit/' . $r->rid, array('query' => 'destination=' . $dest));
    
    $t = ($priorities[$r->page_priority]) ? $priorities[$r->page_priority] : $r->page_priority;
    $row[] = l($t, 'node/' . $r->nid . '/edit', array('query' => 'destination=' . $dest));
    if($kwresearch_exists) {
      $query = 'destination=' . $dest;
      if(!isset($r->kwr_kid)) {
        $query .= '&keyword=' . $r->keyword;
        $t = t('x');
      }
      else if($r->keyword_priority == -1) {
        $t = t('-'); 
      } else {
        $t = ($priorities[$r->keyword_priority]) ? $priorities[$r->keyword_priority] : $r->keyword_priority;
      }
      $row[] = l($t, 'admin/content/kwresearch/keyword_list/edit/' . $r->kwr_kid, array('query' => $query));
    } 
    $row[] = ($attributes['title']) ?  $attributes['title'] : '[node title]';
    $row[] = ($r->sync) ?  'Y' : 'N';
    $row[] = $ops;
    
    $rows[] = array('data' =>
      $row,
      // Attributes for tr
      'class' => "linkintel"
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No request were found.'), 'colspan' => count($header)));
  }  

  $output .= theme('table', $header, $rows, array('id' => 'linkintel-request'));
  $output .= theme('pager', NULL, 100, 0);
  
  $output .= l(t('Add request'), 'admin/content/linkintel/request/edit/', array('query' => 'destination=' . $dest . '&nid=' . $node->nid));
  return $output;
}

/**
 * Generates table of keywords associated with a pages
 * 
 * @param int|str|nodeobj $pid - page id; the nid (int) or path (str) of the page 
 */
function linkintel_requests_page($rand = 0) {
  drupal_set_title('Requests');
  $kwresearch_exists = module_exists('kwresearch');
  $filter = linkintel_requests_build_filter_query();
//dsm($filter);

  $output .= drupal_get_form('linkintel_requests_filter_form');
  
  $header = array();
  $header[] = array('data' => t('Keyword'), 'field' => 'k.keyword');
  $header[] = array('data' => t('From'), 'field' => 'pid');
  $header[] = array('data' => t('Score'), 'field' => 'score', 'sort' => 'desc');
  $header[] = array('data' => t('Request'), 'field' => 'r.priority');
  $header[] = array('data' => t('Page'), 'field' => 'p.priority');
  if($kwresearch_exists) {
    $header[] = array('data' => t('Keyword'), 'field' => 'keyword_priority'); 
  }  
  $header[] = array('data' => t('Title'));
  $header[] = array('data' => t('Sync'), 'field' => 'r.sync');
  $header[] = array('data' => t('Ops'));
  
  if($rand) {
    $rand_sql = linkintel_get_autogen_rand_sql();
  }
  if($kwresearch_exists) {
    $div = ($rand_sql) ? 62500 : 1250;
    $select = "kwr.kid AS kwr_kid, kwr.priority AS keyword_priority, (r.priority * IF(p.priority >= 0, p.priority, 50) * IF(kwr.priority >= 0, kwr.priority, 50) $rand_sql / $div) AS score";
    $join = "LEFT JOIN {kwresearch_keyword} kwr ON kwr.keyword = k.keyword";
  }
  else {
    $div = ($rand_sql) ? 1250 : 25;
    $select = "(r.priority * IF(p.priority >= 0, p.priority, $rand_sql / $div) AS score";
  }
  
  $sql = "
    SELECT r.*, IF(r.nid > 0, r.nid, r.path) AS pid, k.keyword, p.priority AS page_priority, 
    $select
    FROM {linkintel_request} r
    JOIN {linkintel_keyword} k ON k.kid = r.kid
    LEFT JOIN {linkintel_page} p ON p.nid = r.nid 
    $join
  ";
  $tablesort = tablesort_sql($header);
  if (!empty($filter['where'])) {
    $sql = $sql ." WHERE " . $filter['where'] . $tablesort;
    $result = pager_query($sql, 100, 0, NULL, $filter['args']);
  }
  else {
    $result = pager_query($sql . $tablesort, 100);
  }

  $priorities = linkintel_get_priority_options();
  $dest = 'admin/content/linkintel/requests/' . $pid;
  $rows = array();
  while ($r = db_fetch_object($result)) {
    $attributes = unserialize($r->attributes);
    $options = array();
    if($r->external) {
      $options['absolute'] = TRUE;
    }
    $ops = l(t('edit'), 'admin/content/linkintel/request/edit/' . $r->rid, array('query' => 'destination=' . $dest));
    $ops .= ' | ' . l(t('delete'), 'admin/content/linkintel/request/delete/' . $r->rid, array('query' => 'destination=' . $dest));
    $row = array();
    $row[] = check_plain($r->keyword);
    $row[] = ($r->pid > 0) ? l('node/' . $r->pid, 'node/' . $r->pid, $options) : l($r->pid, $r->pid, $options);
    $row[] = number_format($r->score);
    $t = ($priorities[$r->priority]) ? $priorities[$r->priority] : $r->priority;
    $row[] = l($t, 'admin/content/linkintel/request/edit/' . $r->rid, array('query' => 'destination=' . $dest));
    
    $t = ($priorities[$r->page_priority]) ? $priorities[$r->page_priority] : $r->page_priority;
    $row[] = l($t, 'node/' . $r->nid . '/edit', array('query' => 'destination=' . $dest));
    if($kwresearch_exists) {
      $query = 'destination=' . $dest;
      if(!isset($r->kwr_kid)) {
        $query .= '&keyword=' . $r->keyword;
        $t = t('x');
      }
      else if($r->keyword_priority == -1) {
        $t = t('-'); 
      } else {
        $t = ($priorities[$r->keyword_priority]) ? $priorities[$r->keyword_priority] : $r->keyword_priority;
      }
      $row[] = l($t, 'admin/content/kwresearch/keyword_list/edit/' . $r->kwr_kid, array('query' => $query));
    } 
    $row[] = ($attributes['title']) ?  $attributes['title'] : '[node title]';
    $row[] = ($r->sync) ?  'Y' : 'N';
    $row[] = $ops;
    
    $rows[] = array('data' =>
      $row,
      // Attributes for tr
      'class' => "linkintel"
    );
  }

  if (!$rows) {
    $rows[] = array(array('data' => t('No link request were found.'), 'colspan' => count($header)));
  }  

  $output .= theme('table', $header, $rows, array('id' => 'linkintel-request'));
  $output .= theme('pager', NULL, 100, 0);
  
  //$output .= l(t('Add request'), 'admin/content/linkintel/request/edit/', array('query' => 'destination=' . $dest ));
  return $output;
}

function linkintel_requests_build_filter_query() {
  if (empty($_SESSION['linkintel_requests_filter'])) {
    return;
  }

  $filters = linkintel_requests_filters();

  // Build query
  $where = $args = array();
//dsm($_SESSION['kwresearch_overview_filter']);
  foreach ($_SESSION['linkintel_requests_filter'] as $key => $filter) {
//dsm("$key = $filter " . $filters[$key]['type']);
    if($key == 'like_match') {
      continue;
    }
    $filter_where = array();
    if ($filters[$key]['type'] == 'select') {
      foreach ($filter as $value) {
        $filter_where[] = $filters[$key]['where'];
        $args[] = $value;
      }
    }
    elseif ($filter) {      
      if ($key == 'keywords') {        
        if($_SESSION['linkintel_requests_filter']['like_match']) {
          $filter_where[] = $filters['like_match']['where'];
          $args[] = $filter;
        }
        else {
          $keywords = explode(',', $filter);
          $placeholders = implode(',', array_fill(0, count($keywords), '"%s"'));
          $filter_where[] = str_replace('"%s"', $placeholders, $filters[$key]['where']);
          foreach($keywords AS $keyword) {
            $keyword = trim(strtolower($keyword));
            $args[] = $keyword;
          }
        }
      }
      else if($key == 'from') {
        // treat as nid input
        $a = explode('/', $filter);
        if(($a[0] == 'node' && is_numeric($a[1])) || ($filter > 0)) {
          $nid = (($a[0] == 'node' && is_numeric($a[1])))? $a[1] : $filter;
          $filter_where[] = 'r.nid = %d';
          $args[] = $nid;
        }
        else {
          $filter_where[] = $filters[$key]['where'];
          $args[] = $filter;         
        }
      }
      else {
        $filter_where[] = $filters[$key]['where'];
        $args[] = $filter;
      }      
    }
    if (!empty($filter_where)) {
      $where[] = '('. implode(' OR ', $filter_where) .')';
    }
  }
  $where = !empty($where) ? implode(' AND ', $where) : '';

  return array(
    'where' => $where,
    'args' => $args,
  );
}

/**
 * Return form for site keyword list filters.
 *
 */
function linkintel_requests_filter_form() {
  $session = &$_SESSION['linkintel_requests_filter'];
  $session = is_array($session) ? $session : array();
  $filters = linkintel_requests_filters();

  $form['filters'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filter requests'),
    '#theme' => 'dblog_filters',
    '#collapsible' => TRUE,
    '#collapsed' => empty($session),
  );
  foreach ($filters as $key => $filter) {
    if ($filter['type'] == 'textfield') {
      $form['filters']['status'][$key] = array(
        '#title' => $filter['title'],
        '#type' => 'textfield',
        '#size' => ($filter['size'])? $filter['size']:8,
      );    
    }
    elseif ($filter['type'] == 'select') {
      $form['filters']['status'][$key] = array(
        '#title' => $filter['title'],
        '#type' => 'select',
        '#multiple' => TRUE,
        '#size' => 6,
        '#options' => $filter['options'],
      );
    }
    elseif ($filter['type'] == 'checkbox') {
      $form['filters']['status'][$key] = array(
        '#title' => $filter['title'],
        '#type' => 'checkbox',
        '#return_value' => $filter['return_value'],
      );
    }
    elseif ($filter['type'] == 'markup') {
      $form['filters']['status'][$key] = array(
        '#type' => $filter['type'],
        '#value' => $filter['value'],
      );       
    }
    if (!empty($session[$key])) {
      $form['filters']['status'][$key]['#default_value'] = $session[$key];
    }
  }

  $form['filters']['buttons']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
  );
  if (!empty($session)) {
    $form['filters']['buttons']['reset'] = array(
      '#type' => 'submit',
      '#value' => t('Reset')
    );
  }

  return $form;
}

/**
 * Returns filters for site keywords report
 */
function linkintel_requests_filters() {
  $filters = array();

  $options = linkintel_get_priority_options();
  $filters['keywords'] = array(
    'title' => t('Keywords'),
    'type' => 'textfield',
    'size' => 60,
    'description' => t('Enter a comma seperated list of keywords to filter by exact match.'),
    'where' => 'k.keyword IN ("%s")',
  ); 
  $filters['like_match'] = array(
    'title' => t('Like match'),
    'type' => 'checkbox',
    'return_value' => 1,
    'where' => 'k.keyword LIKE "%%%s%%"',
  );
  $filters['from'] = array(
    'title' => t('From'),
    'type' => 'textfield',
    'size' => 60,
    'where' => 'path = "%s"',
    'description' => t('Input a path or nid.'),
  ); 
  $filters['request_priority'] = array(
    'title' => t('Request priority'),
    'type' => 'select',
    'where' => 'r.priority = %d',
    'options' => $options,
  );
  $filters['page_priority'] = array(
    'title' => t('Request priority'),
    'type' => 'select',
    'where' => 'p.priority = %d',
    'options' => $options,
  );
  $filters['keyword_priority'] = array(
    'title' => t('Request priority'),
    'type' => 'select',
    'where' => 'kwr.priority = %d',
    'options' => $options,
  );
   
  $filters['sync'] = array(
    'title' => t('Sync'),
    'type' => 'checkbox',
    'return_value' => 1,
    'where' => 'r.sync = %d',
  ); 

  return $filters;
}

/**
 * Validate result from site keyword report filter form.
 */
function linkintel_requests_filter_form_validate($form, &$form_state) {
  if ($form_state['values']['op'] == t('Filter') 
    && empty($form_state['values']['keywords'])
    && empty($form_state['values']['from']) 
    && empty($form_state['values']['request_priority'])
    && empty($form_state['values']['page_priority'])
    && empty($form_state['values']['keyword_priority'])
    && empty($form_state['values']['sync'])
  ) {
    form_set_error('type', t('You must select something to filter by.'));
  }
}

/**
 * Process result from site keywords report filter form.
 */
function linkintel_requests_filter_form_submit($form, &$form_state) {
  $op = $form_state['values']['op'];
  $filters = linkintel_requests_filters();
  switch ($op) {
    case t('Filter'):
      foreach ($filters as $name => $filter) {
        if (isset($form_state['values'][$name])) {
          $_SESSION['linkintel_requests_filter'][$name] = $form_state['values'][$name];
        }
      }
      break;
    case t('Reset'):
      $_SESSION['linkintel_requests_filter'] = array();
      break;
  }
  return 'admin/content/linkintel/requests';
}

/**
 * Generages page keyword edit form
 * 
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 * @param int $kid - keyword id
 */
function linkintel_request_edit_page($rid = '') {
  if($rid > 0) {
    $request_obj = linkintel_load_request($rid);
  }
  if ($request_obj->rid) {
    drupal_set_title(t('Edit request'));
  }
  else {
    drupal_set_title(t('Add request'));
    $request_obj = new stdClass();
    if($_GET['nid']) {
      $request_obj->nid = $_GET['nid'];
    }
  }  
  $output .= drupal_get_form('linkintel_request_edit_form', $request_obj);
  return $output;
}

/**
 * Returns page keyword edit form
 * 
 * @param arr $form_state
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 * @param str $keyword
 */

function linkintel_request_edit_form($form_state, $request_obj = stdClass) {
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => ($request_obj->rid) ? $request_obj->rid : ''
  );
  if($request_obj->nid || $request_obj->path) {
    $url0 = ($request_obj->path) ? $request_obj->path : 'node/' . $request_obj->nid;
    $url1 = url($url0);
    $form['page'] = array(
        '#title' => t('Page'),
        '#type' => 'item',
        '#value' =>  l($url1, $url0),
    );
    $form['nid'] = array(
      '#type' => 'value',
      '#value' => $request_obj->nid
    );
    $form['path'] = array(
      '#type' => 'value',
      '#value' => $request_obj->path
    );
  }
  else {
    $form['nid'] = array(
      '#type' => 'textfield',
      '#title' => t('Node id'),
      '#size' => 8, 
      '#required' => TRUE,
      '#description' => t('Enter the nid you want to associate this request.'),
    );    
  }
  if ($request_obj->kid) {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => $request_obj->kid
    );
    $form['keyword'] = array(
      '#type' => 'item',
      '#title' => t('Keyword'), 
      '#value' => $request_obj->keyword,
    );    
  }
  else {
    $form['kid'] = array(
      '#type' => 'value',
      '#value' => 0,
    );
    $form['keyword'] = array(
      '#type' => 'textfield',
      '#title' => t('Keyword'), 
      '#required' => TRUE,
    );  
  }
  
  $options = linkintel_get_priority_options();
  $form['priority'] = array(
    '#type' => 'select',
    '#title' => t('Priority'), 
    '#options' => $options,
    '#default_value' => ($request_obj->priority) ? $request_obj->priority:50,
    '#description' => t('Use to set how important the keyword target is for the page. If unsure, just set to "Standard".'),
  ); 
  $form['sync'] = array(
    '#type' => 'checkbox',
    '#title' => t('Sync'), 
    '#return_value' => 1,
    '#default_value' => $request_obj->sync,
    '#description' => t('Check if you want this request to be automatically synced with page keywords.'),
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $request_obj->title,
    '#description' => t('For inline links, used as the anchor title attribute. Used as the anchor text for sidebar links. Leave blank to use the node title.'),
  ); 
    
  $form['save'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  
  return $form;
}

/**
 * Validates the page keyword edit form
 * 
 * @param $form
 * @param $form_state
 */
function linkintel_request_edit_form_validate($form, &$form_state) {
  $node = node_load($form_state['values']['nid']);
  if(!$node->nid) {
    form_set_error('nid', t('A node does not exist for the nid. Please provide a valid nid'));
  }
}

/**
 * Processes the page keyword edit form
 * 
 * @param $form
 * @param $form_state
 */
function linkintel_request_edit_form_submit($form, &$form_state) {

  $request_obj = new stdClass();
  $request_obj->mode = $form_state['values']['mode'];
  $request_obj->rid = $form_state['values']['rid'];
  $request_obj->nid = $form_state['values']['nid'];
  $request_obj->kid = $form_state['values']['kid'];
  $request_obj->keyword = $form_state['values']['keyword'];
  $request_obj->priority = $form_state['values']['priority'];
  $request_obj->sync = $form_state['values']['sync'];
  $request_obj->title = $form_state['values']['title'];
  // embed in node body
  linkintel_save_request($request_obj);
  
  drupal_set_message(t('A link request was saved for nid <em>@nid</em> and keyword <em>@keyword</em>.', 
    array(
      '@nid' => $request_obj->nid,
      '@keyword' => $request_obj->keyword
    )),
    'status'
  );
}

function linkintel_request_delete_page($rid) {
  drupal_set_title(t('Delete link request'));
  $request = linkintel_load_request($rid);
  if(!$request->rid) {
    drupal_set_message(t('A link request with ID @id was not found.', array('@id' => $request->rid)), 'error');
    return '';
  }
  $output .= drupal_get_form('linkintel_request_delete_form', $request);
  return $output;
}

/**
 * Generages page keyword edit form
 * 
 * @param int|str $pid - page id; the nid (int) or path (str) of the page
 * @param int $kid - keyword id
 */
function linkintel_request_delete_form($form_state, $request) {
  $form['message'] = array(
  '#type' => 'markup',
  '#value' => t('Are you sure you want to delete the link request for the keyword <em>@keyword</em> to node <em>@nid</em>?',
      array(
        '@keyword' => $request->keyword,
        '@nid' => $request->nid,
      )
    ),
  );
  
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $request->rid,
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Delete'),
  );
  $form['cancel'] = array(
      '#type' => 'markup',
      '#value' => l(t('Cancel'), $_GET['destination']),
  );
  
  return $form;
}

function linkintel_request_delete_form_submit($form, &$form_state) {

  $request_obj = linkintel_load_request($form_state['values']['rid']);

  linkintel_delete_request($request_obj);
  
  drupal_set_message(t('The link request was deleted for nid <em>@nid</em> and keyword <em>@keyword</em>.', 
    array(
      '@nid' => $request_obj->nid,
      '@keyword' => $request_obj->keyword
    )),
    'status'
  );
}